#include "tutte.h"
#include "filegraphutils.h"
#include <string.h>
#include <sys/times.h>
#include <sys/time.h>
#include <time.h>
#define USAGE "Ejemplo de uso : tutte [-i | -o | -io ] [m | l | t] [infile] [outfile]\n"

#define HELPTEXT \
"Escribe el polinomio de Tutte correspondiente a un conjunto de grafos pasados por fichero o entrada estándar en formato Graph6 o Sparse6\n\
        \n\
        -i  : Especifica que los grafos deben ser pasados por fichero.\n\
        -o  : Para especificar que se generen los grafos en un fichero de salida.\n\
        -io  : La aplicación asume que existen ficheros de entrada y salida.\n\
        -m  : Los grafos se imprimen como matrices de adyacencias.\n\
        -l  : Se muestran los grafos como listas de adyacencias.\n\
        -t  : Imprime los grafos como la parte triangular superior de la matriz de adyacencias.\n\n\
        -q  : Durante la ejecución independiente de la herramienta permite finalizar la ejecución a petición del usuario\n\
        \n\
        \n"

#define CLK_TCK 100

#define CPUTIME (times(&timebuffer),\
              (double)(timebuffer.tms_utime + timebuffer.tms_stime) / CLK_TCK)

double timeval_diff(struct timeval *a, struct timeval *b)
{
  return
    (double)(a->tv_sec + (double)a->tv_usec/1000000) -
    (double)(b->tv_sec + (double)b->tv_usec/1000000);
}



int main(int argc, char *argv[])
{
        char * cadena;
        char  ruta[256];
        char  ruta2[256];
        FILE* f;
        FILE* f2;
        char entries1[4];
        char entries2[3];
        bool outMatrix = false;
        bool outList = false;
        bool outUpper = false;
        bool outFile = false;
        static struct tms timebuffer;
        double t;


        clock_t t_ini, t_fin;
        clock_t t_ini1, t_fin1;
        struct timeval tt_ini, tt_fin;
        struct timeval tt_ini1, tt_fin1;
        double secs,secst,secst1;
        double secs1;

        if(argc>1){
             if(argc==4){
                 strncpy(entries1,argv[1],4);
                 entries1[3]='\0';
                 if(strcmp(entries1,"-io")==0){
                     if(strlen(argv[1])>3){
                         if(argv[1][3]=='m')
                             outMatrix = true;
                         else if (argv[1][3]=='l')
                             outList = true;
                         else if (argv[1][3]=='t')
                             outUpper = true;
                         else{
                             printf("Error en parámetros de entrada\n");
                             printf("%s",USAGE);
                             return 1;
                         }
                     }
                     strcpy(ruta,argv[2]);
                     strcpy(ruta2,argv[3]);
                     f2 = fopen (ruta2,"w");
                     if(f2==NULL){
                         printf("La ruta del fichero a escribir es incorrecta o no se ha podido crear el fichero\n");
                         return 1;
                     }
                     f = openFileGraph(ruta);
                     if(f==NULL){
                         printf("La ruta del fichero de entrada es incorrecta o no se ha podido leer fichero\n");
                         return 1;
                     }
                     printf("Tutte para fichero de entrada %s se escribirá en %s\n",ruta,ruta2);

                 }
                 else{
                     printf("Error en parámetros de entrada\n");
                     printf("%s",USAGE);
                     return 1;
                 }
             }
             else if (argc==3){
                 strncpy(entries2,argv[1],3);
                 entries2[2]='\0';
                 if(strcmp(entries2,"-i")==0){
                     if(strlen(argv[1])>2){
                         if(argv[1][2]=='m')
                             outMatrix = true;
                         else if (argv[1][2]=='l')
                             outList = true;
                         else if (argv[1][2]=='t')
                             outUpper = true;
                         else{
                             printf("Error en parámetros de entrada\n");
                             printf("%s",USAGE);
                             return 1;
                         }
                     }
                     strcpy(ruta,argv[2]);
                     f2 =stdout;
                     f = openFileGraph(ruta);
                     if(f==NULL){
                         printf("La ruta del fichero de entrada es incorrecta o no se ha podido leer fichero\n");
                         return 1;
                     }
                     printf("Tutte para fichero de entrada %s\n",ruta);
                 }
                 else if(strcmp(entries2,"-o")==0){
                     outFile = true;
                     if(strlen(argv[1])>2){
                         if(argv[1][2]=='m')
                             outMatrix = true;
                         else if (argv[1][2]=='l')
                             outList = true;
                         else if (argv[1][2]=='t')
                             outUpper = true;
                         else{
                             printf("Error en parámetros de entrada\n");
                             printf("%s",USAGE);
                             return 1;
                         }
                     }
                     strcpy(ruta2,argv[2]);
                     f2 = fopen (ruta2,"w");
                     if(f2==NULL){
                         printf("La ruta del fichero a escribir es incorrecta o no se ha podido crear el fichero\n");
                         return 1;
                     }
                     f=stdin;
                     printf("Tutte se escribirá en el fichero de salida %s\n",ruta2);
                 }
                 else{
                     printf("Error en parámetros de entrada\n");
                     printf("%s",USAGE);
                     return 1;
                 }

            }
            else if(argc==2){
             if(strcmp(argv[1],"--help")==0 || strcmp(argv[1],"-help")==0){
                 printf("\n");
                 printf("%s",USAGE);
                 printf("\n");
                 printf("%s",HELPTEXT);
                 return 1;
             }
             else if(strcmp(argv[1],"-l")==0){
                 outList = true;
                 f2 = stdout;
                 f = stdin;
             }
             else if(strcmp(argv[1],"-m")==0){
                 outMatrix = true;
                 f2 = stdout;
                 f = stdin;
             }
             else if(strcmp(argv[1],"-t")==0){
                 outUpper = true;
                 f2 = stdout;
                 f = stdin;

             }
             else{
                 printf("Error en número de parámetros\n");
                 printf("%s",USAGE);
                 return 1;
                 }
            }
            else{
                printf("Error en número de parámetros\n");
                printf("%s",USAGE);
                return 1;
            }


        }
         else{
             printf("Datos por entrada estándar\n");
             f2 = stdout;
             f = stdin;


             }



        if(f==NULL)
            return 1;
        int i=0;
        long j=0;
        cadena=getGraphLine(f);

        gettimeofday(&tt_ini, NULL);

        while (cadena!=NULL){
           strncpy(entries2,cadena,2);
           entries2[2]='\0';
           if(strcmp(entries2,"-q")==0){
               printf("\nFinalización de la ejecución a petición de usuario \n\n");
               fclose(f);
               fclose(f2);
               return 1;
           }

           i=graphsize(cadena);
           multigraph * grafo = new multigraph(i);
           polynom * pol = new polynom(1);
           printf("Grafo %ld a procesar: \n\n",j);
           if(outList!=true && outMatrix!=true && outUpper!=true){
               fprintf(f2,"%s",cadena);
               fprintf(f2,"\n");
           }

           stringToMultigraph(cadena,grafo);
           if(outList == true)
               grafo->printGraphAsAdjListInFile(f2);           
           else if (outMatrix == true)
               grafo->printGraphAsAdjMatrixInFile(f2);
           else if (outUpper == true)
               grafo->printGraphAsUpperMatrixInFile(f2);
           printf("\nProcesando Tutte para grafo %ld\n\n",j);
           gettimeofday(&tt_ini1, NULL);
           tutteAlgorithm(grafo,pol);
           if (!outFile)
               printf("Polinomio de Tutte correspondiente al grafo %ld : \n\n",j);
           else
               printf("Polinomio de Tutte correspondiente al grafo %ld guardado en %s \n\n",j,ruta2);
           pol->printPolynomInFile(f2);
           if(outList == true || outMatrix == true || outUpper == true)
               fprintf(f2,"\n");




           gettimeofday(&tt_fin1, NULL);
           secst1 = timeval_diff(&tt_fin1, &tt_ini1);

           printf("\nTiempo para el cálculo del grafo %ld : %.16g segundos\n\n",j, secst1);
           j++;

           grafo->destroyGraph();
           delete grafo;              
           pol->destroyPolynom();
           delete pol;
           cadena=getGraphLine(f);

        }


        gettimeofday(&tt_fin, NULL);

        secst = timeval_diff(&tt_fin, &tt_ini);

        printf("Tiempo total de ejecución : %.16g segundos\n", secst );


        fclose(f);
        fclose(f2);

        return 1;

}
