#include "polynom.h"


polynom::polynom()
{

    this->n = MAXP;
    this->index=makeDinMat(this->n);
}


polynom::polynom(int m)
{
    this->n = m;
    this->index=makeDinMat(this->n);
}

void polynom::setComponent(int x, int y, int value){
    this->index[x][y]=value;
}


int polynom::getComponent(int x, int y){
    return this->index[x][y];
}

int polynom::getOrder(){
    return this->n;
}
void polynom::cleanPolynom(){
    int i,j;

    for (i=0;i<this->n;i++)
        for(j=0;j<this->n;j++)
            this->index[i][j] = 0;
}
void polynom::destroyPolynom(){
    destroyMat(this->index,this->n);
    this->index=NULL;
}
void polynom::addPolynom (polynom * a){

    int i,j,m,orderA, orderB;
    orderA = a->getOrder();
    orderB = this->getOrder();
    polynom * result = new polynom(orderA > orderB ? orderA : orderB);


    m = result->getOrder();

    for (i=0;i<m;i++){
        for(j=0;j<m;j++){
            if(i>=a->getOrder() || j>=a->getOrder()){
                result->setComponent (i,j,this->getComponent(i,j));
            }
            else if (i>=this->getOrder() || j>=this->getOrder()){
                result->setComponent (i,j,a->getComponent(i,j));
            }
            else{
                result->setComponent (i,j,a->getComponent(i,j) + this->getComponent(i,j));
            }
        }
    }

    this->copyPolynom(result);
    result->destroyPolynom();
    delete result;
}
void polynom::copyPolynom(polynom * g){
    int i,j;
    destroyMat(this->index,this->n);
    this->n=g->getOrder();
    this->index=makeDinMat(this->n);
    for(i=0;i<this->n;i++){
        for(j=0;j<this->n;j++){
            this->index[i][j]=g->getComponent(i,j);
        }
    }
}
void polynom::printPolynom(){
    int i,j,m;
    m = this->getOrder();
    for (i=0;i<m;i++){
        for(j=0;j<m;j++){
            if(this->getComponent(i,j)!=0){
                if(i==0 && j==0){
                    printf("%d",this->getComponent(i,j));
                }
                else if (i==0){
                    printf("+%dY^%d",this->getComponent(i,j),j);
                }
                else if (j==0){
                    printf("+%dX^%d",this->getComponent(i,j),i);
                }
                else{
                    printf("+%dX^%dY^%d",this->getComponent(i,j),i,j);
                }
            }
            else if (i==0 && j==0)
                printf("%d",this->getComponent(i,j));
        }
    }
    printf("\n");
}
void polynom::multX(){

    int i,j,m;
    bool foundInLastRow = false;
    m = this->getOrder();
    for (i=0;i<m;i++){
        if(this->getComponent(m-1,i) != 0){
            foundInLastRow = foundInLastRow || true;
        }
    }

    if (!foundInLastRow){
        polynom * auxAmbito = new polynom (m);
        for(i=0;i<m-1;i++){
            for(j=0;j<m;j++){
                    auxAmbito->setComponent(i+1,j,this->getComponent(i,j));
            }
        }
        this->copyPolynom(auxAmbito);
        auxAmbito->destroyPolynom();
        delete auxAmbito;
    }
    else{
        polynom * auxAmbito = new polynom (m+1);
        for(i=0;i<m;i++){
            for(j=0;j<m;j++){
                    auxAmbito->setComponent(i+1,j,this->getComponent(i,j));
            }
        }
        this->copyPolynom(auxAmbito);
        auxAmbito->destroyPolynom();
        delete auxAmbito;
    }

}

void polynom::multY(){

    int i,j,m;
    bool foundInLastColum = false;
    m = this->getOrder();
    for (i=0;i<m;i++){
        if(this->getComponent(i,m-1) != 0){
            foundInLastColum = foundInLastColum || true;
        }
    }

    if (!foundInLastColum){
        polynom * auxAmbito = new polynom (m);
        for(i=0;i<m;i++){
            for(j=0;j<m-1;j++){
                    auxAmbito->setComponent(i,j+1,this->getComponent(i,j));
            }
        }
        this->copyPolynom(auxAmbito);
        auxAmbito->destroyPolynom();
        delete auxAmbito;
    }
    else{
        polynom * auxAmbito = new polynom (m+1);
        for(i=0;i<m;i++){
            for(j=0;j<m;j++){
                    auxAmbito->setComponent(i,j+1,this->getComponent(i,j));
            }
        }
        this->copyPolynom(auxAmbito);
        auxAmbito->destroyPolynom();
        delete auxAmbito;
    }


}

void polynom::printPolynomInFile(FILE * f){
    int i,j,m;
    m = this->getOrder();
    for (i=0;i<m;i++){
        for(j=0;j<m;j++){
            if(this->getComponent(i,j)!=0){
                if(i==0 && j==0){
                    fprintf(f,"%d",this->getComponent(i,j));
                }
                else if (i==0){
                    fprintf(f,"+%dY^%d",this->getComponent(i,j),j);
                }
                else if (j==0){
                    fprintf(f,"+%dX^%d",this->getComponent(i,j),i);
                }
                else{
                    fprintf(f,"+%dX^%dY^%d",this->getComponent(i,j),i,j);
                }
            }
            else if (i==0 && j==0)
                fprintf(f,"%d",this->getComponent(i,j));
        }
    }
    fprintf(f,"\n");
}

